"""Data-readers for BioLogic"""

from ecdh.log import LOG

import numpy as np
import pandas as pd


def mpt_biologic_to_vq_old(filepath):
    import numpy as np
    f = open(filepath, 'r') #open the filepath for the mpt file
    # now we skip all the data in the start and jump straight to the dense data
    headerlines = 0
    for line in f:
        if "Nb header lines :" in line:
            headerlines = int(line.split(':')[-1])
            break #breaks for loop when headerlines is found
    
    for line in f:
        if "Characteristic mass :" in line:
            active_mass = float(line.split(':')[-1][:-3].replace(',', '.'))/1000
            LOG.success("Active mass found in file to be: " + str(active_mass) + "g")
            break #breaks loop when active mass is found

    #Skip all headerlines
    for i in range(headerlines):
        f.readline()


    #Put all the data in one big list of lines
    data = f.readlines()

    #Spawning massive data arrays for Potential, Charge and Cycle num
    Ewe_arr = np.zeros(len(data))
    Q_chg_dischg_arr = np.zeros_like(Ewe_arr)
    Cycle_arr = np.zeros_like(Ewe_arr)
    ox_red = np.zeros_like(Ewe_arr)

    #Putting all data (potential, capacity, cyclenum) in arrays
    i=-1
    for line in data: #Looping over each line in the dense data
        
        i+=1
        line_lst = line.split() #Splitting the line to a list
        Cycle_arr[i] = int(float(line_lst[-2].replace(',', '.'))) #The integer of the last element on the line as cycle added to array of cycles
        Ewe_arr[i] = float(line_lst[11].replace(',', '.')) #line elem 11 added to Ewe arr as float
        Q_chg_dischg_arr[i] = float(line_lst[19].replace(',', '.')) / active_mass
        ox_red[i] = int(float(line_lst[1].replace(',', '.')))
        

    #Creating list with indexes of new cycle starts
    list_of_cycleindexes = [0]
    list_of_reversalindexes = []
    for i in range(len(Cycle_arr)-1):
        if Cycle_arr[i]+1 == Cycle_arr[i+1]: #Adding the index where a new cycle starts (stop discharge, start charge)
            list_of_cycleindexes.append(i+1)
        if ox_red[i] == ox_red[i+1]+1: #Adding the index where the charge stops and the discharge starts
            list_of_reversalindexes.append(i+1)

    charges = []
    discharges = []
    indx = list_of_cycleindexes
    [indx.append(i) for i in list_of_reversalindexes]
    indx.sort()

    for i in range(len(indx) - 1):
        if (i%2) == 0: #even index: charge cycle
            charges.append((Ewe_arr[indx[i]:indx[i+1]], Q_chg_dischg_arr[indx[i]:indx[i+1]])) #Inserting tuple of two arrays sliced to be one cycle of voltages and capacities.
        else: #then discharge
            discharges.append((Ewe_arr[indx[i]:indx[i+1]], Q_chg_dischg_arr[indx[i]:indx[i+1]]))
    return charges, discharges
    

def read_mpt(filepath):
    """
    Reads an mpt file to a pandas dataframe
    
    .MPT format:
        mode column: 1=Galvanostatic, 2=Linear Potential Sweep, 3=Rest

    """
    modes = {1: "Galvanostatic", 2 : "Linear Potential Sweep", 3 : "Rest"}

    #with open(filepath, 'r', encoding= "iso-8859-1") as f:  #open the filepath for the mpt file
    #    lines = f.readlines()
    with open(filepath, errors = 'ignore') as f:
        lines = f.readlines()

    # now we skip all the data in the start and jump straight to the dense data
    headerlines = 0
    for line in lines:
        if "Nb header lines :" in line:
            headerlines = int(line.split(':')[-1])
            break #breaks for loop when headerlines is found
    
    for line in lines:
        if "Characteristic mass :" in line:
            active_mass = float(line.split(':')[-1][:-3].replace(',', '.'))/1000
            LOG.debug("Active mass found in file to be: " + str(active_mass) + "g")
            break #breaks loop when active mass is found


    big_df = pd.read_csv(filepath, header=headerlines-1, sep="\t", encoding = "ISO-8859-1")
    LOG.debug("Dataframe column names: {}".format(big_df.columns))

    # Start filling dataframe
    df = big_df[['mode', 'time/s', 'Ewe/V', '<I>/mA', 'cycle number', 'ox/red']]

    # If it's galvanostatic we want the capacity
    mode = df['mode'].value_counts() #Find occurences of modes
    mode = mode[mode.index != 3] #Remove the count of modes with rest (if there are large rests, there might be more rest datapoints than GC/CV steps)
    mode = mode.idxmax()    # Picking out the mode with the maximum occurences
    LOG.debug("Found cycling mode: {}".format(modes[mode]))

    if mode == 1:
        df = df.join(big_df['Capacity/mA.h'])
        df.rename(columns={'Capacity/mA.h': 'capacity/mAhg'}, inplace=True)
        df['capacity/mAhg'] = pd.to_numeric(df['capacity/mAhg'].str.replace(',','.'))

    # Replace , by . and make numeric from strings. Mode is already interpreted as int.
    df['time/s'] = pd.to_numeric(df['time/s'].str.replace(',','.'))
    df['Ewe/V'] = pd.to_numeric(df['Ewe/V'].str.replace(',','.'))
    df['<I>/mA'] = pd.to_numeric(df['<I>/mA'].str.replace(',','.'))
    df['cycle number'] = pd.to_numeric(df['cycle number'].str.replace(',','.')).astype('int32')
    df.rename(columns={'ox/red': 'charge'}, inplace=True)
    df['charge'].replace({1: True, 0: False}, inplace = True)

    #Adding attributes must be the last thing to do since it will not be copied when doing operations on the dataframe
    df.experiment_mode = mode
    
    return df
    

def custom_EC_export(filepath):
    import numpy as np
    with open(filepath, 'r') as f: #open the filepath for the mpt file
        f.readline()
        lines = f.readlines()

    #Ox/red-col = 1
    #Ewe-col = 4
    #Cap-col = 5
    #Cyc-col = 0
    #Cur-col = 3
    #time-col = 2
    charges = []
    discharges = []
    newox = False
    oldox = eval(lines[0].split()[1])
    oldcap = 0
    t_prev = 0
    t_cycstart = 0
    tmp_cyc_E = []
    tmp_cyc_C = []
    Cap_cum = []
    for line in lines:
        if oldox == eval(line.split()[1]):
            tmp_cyc_E.append(eval(line.split()[4]))
            tmp_cyc_C.append(eval(line.split()[5]))
            
            #Manual capacity calculation
            I_cur = eval(line.split()[3])
            t_cur = eval(line.split()[2])/3600 - t_cycstart
            timedelta = t_cur - t_prev
            Cap_cum.append(abs(oldcap + I_cur * timedelta))
            t_prev = t_cur
            oldcap += I_cur * timedelta

        else:
            #print("End-Halfcycle-triggered")
            if oldox == 1: #Charge cycle
                charges.append((np.array(tmp_cyc_E), np.array(Cap_cum))) # Making the lists into arrays and putting them in a tuple as a charge cycle
            elif oldox == 0: #Discharge cycle
                discharges.append((np.array(tmp_cyc_E), np.array(Cap_cum)))
            
            # Resetting the temporary lists
            tmp_cyc_E = []
            tmp_cyc_C = []
            Cap_cum = []
            t_prev = 0
            oldcap = 0
            # Setting the new ox status
            oldox = eval(line.split()[1])
            # Must remember to add this line's values!
            tmp_cyc_E.append(eval(line.split()[4]))
            tmp_cyc_C.append(eval(line.split()[5]))

            t_cycstart = eval(line.split()[2])/3600 # Resetting time of start of cycle
            #Manual capacity calculation
            I_cur = eval(line.split()[3])
            t_cur = eval(line.split()[2])/3600 - t_cycstart
            timedelta = t_cur - t_prev
            Cap_cum.append(oldcap + I_cur * timedelta)
            t_prev = t_cur
            oldcap += I_cur * timedelta

    # Adding the last data to arrays if the file ends
    if oldox == 1: #Charge cycle
        LOG.debug("Datafile ended with a Charge")
        charges.append((np.array(tmp_cyc_E), np.array(Cap_cum))) # Making the lists into arrays and putting them in a tuple as a charge cycle
    elif oldox == 0: #Discharge cycle
        LOG.debug("Datafile ended with a Discharge")
        discharges.append((np.array(tmp_cyc_E), np.array(Cap_cum)))

    return charges, discharges  



import os
from ecdh.log import LOG

def make_toml(folder):
    files = []
    extlist = ["mpt", "csv", "txt", "xlsx"]


    #Loop over all elements in current directory
    for entry in os.listdir(folder):
        # If it is a file and has the correct extension
        if os.path.isfile(entry) and entry.split(".")[-1] in extlist:
            # Add it to filelist
            files.append(entry)

    toml_str = """# Data path and active mass (set to 1 if data is normalized with regards to active mass)
files = [\n"""
    if len(files)>0:
        for file in files:
            toml_str += '\t["' + file + '","1.0"],\n'
    else:
        LOG.warning("Could not find any files in the current folder!")
        toml_str += '\t[" ","1.0"],\n'

    toml_str += """]

[settings]
qcplot = true                   # Wether or not to plot capacity vs cycles
percentage = false              # Wether or not to use percentage in capacity vs cycles plot
rawplot = true                  # Wether or not to plot Potential and Current vs time
rawplot_capacity = false        # Wether or not to use cumulative capacity on x-axis
vcplot = false                  # Wether or not to plot voltage curves (either CV og GC depending on data)
dqdvplot = false                # Wether or not to plot dQ/dV plots from the GCPL data
specific_cycles = false         # Will make global limit of cycles, can be range or list of cycles
suptitle = 'Capacity retention' # Title of plot
ylabel = 'Specific capacity [mAh/g]'
xlabel = 'Cycles'
all_in_one = true
"""

    with open("ecdh.toml", "w") as f:
        f.write(toml_str)
        LOG.info("Wrote example configuration to 'ecdh.toml' with %.0f files found"%len(files))


"""Data-readers for Neware"""

from ecdh.log import LOG
def dat_batsmall_to_vq(filename):
    import numpy as np
    data = []
    decode_errors = 0
    decode_error_str = ""
    with open(filename, "r") as f:
        # Skip all non-data lines
        while True:
            line = f.readline()
            if '"V";I:"A";C:"Ah/kg";7' in line:
                break
        # Adding the rest of the readable file to a data array
        while True:
            try:
                line = f.readline()
                if line == '':
                    break
                else:
                    data.append(line)
            except UnicodeDecodeError as e:
                decode_errors += 1
                decode_error_str = e
                continue
        
        if decode_errors > 0:
            LOG.warning("Found %i Unicode Decode errors, thus %i lines of data has been missed. Consider getting a safer method of acquiring data. \nComplete error message: " %(decode_errors, decode_errors) + str(decode_error_str))


    charges = []
    discharges = []
    charge = True
    voltages = []
    capacities = []

    for line in data[:-1]:

        try:
            v = float(eval(line.split(";")[1]))
            q = abs(float(eval(line.split(";")[3])))
            voltages.append(v)
            capacities.append(q)
        except:
            if '"V";I:"A";C:"Ah/kg"' in line and charge == True:
                #LOG.debug("Charge end at: V: %.5f, Q: %.5f" %(v,q))
                charges.append((np.array(voltages), np.array(capacities)))
                charge = False
                voltages = []
                capacities = []
            elif '"V";I:"A";C:"Ah/kg"' in line and charge == False:
                #LOG.debug("Discharge end at: V: %.5f, Q: %.5f" %(v,q))
                discharges.append((np.array(voltages), np.array(capacities)))
                charge = True
                voltages = []
                capacities = []

    return charges, discharges

def read_txt(filepath):
    """
    Reads a txt file to a pandas dataframe

    .txt format:
        line 1: user comment
        header: includes columns TT, U, I, Z1 ++ , corresponding to time, potential, current and cycle number

    Important: In the datafile, in the first line (which is the user description) you must use the keyword CV or GC to tell what type of data it is."""
    import pandas as pd
    import numpy as np
    import os

    # Open file
    with open(filepath, 'r') as f:
        lines = f.readlines()

    # Find out what type of experiment this is
    expmode = 0
    for line in lines:
        if "CV" in line or "Cyclic Voltammetry" in line or "cyclic voltammetry" in line:
            expmode = 2
            break
        elif "GC" in line or "Galvanostatic" in line or "galvanostatic" in line:
            expmode = 1
            break
        else:
            LOG.warning("Cannot find out which type of experiment the file {} is! Please specify either CV or GC in the comment on the top of the file.".format(os.path.basename(filepath)))
            break

    # Find out how many headerlines this file have
    headerlines = 0
    for i,line in enumerate(lines):
        if 'TT' in line and 'U ' in line and 'I ' in line:
            headerlines = i
            break

    # Read all data to a pandas dataframe
    big_df = pd.read_csv(filepath, header = headerlines-1, sep = "\t")

    #Extract useful columns, change the name of the columns, make all numbers numbers.
    df = big_df[['TT [h]', 'U [V]', 'I [mA]', 'Z1 []', 'C [mAh/kg]']]
    df.rename(columns={'TT [h]': 'time/s', 'U [V]': 'Ewe/V', 'I [mA]': '<I>/mA', 'Z1 []':'cycle number', 'C [mAh/kg]':'capacity/mAhg'}, inplace=True)
    df = df.astype({"time/s": float, "Ewe/V": float, "<I>/mA": float, "capacity/mAhg": float, "cycle number": int})
    df['time/s'] = df['time/s'].apply(lambda x: x*3600) #Converting from h to s
    df['capacity/mAhg'] = df['capacity/mAhg'].apply(lambda x: abs(x/1000)) #Convert from mAh/kg to mAh/g
    df['mode'] = expmode
    df['charge'] = True
    df.experiment_mode = expmode
    df.name = os.path.basename(filepath)

    check_df(df)

    return df

def check_df(df):
    """Check if the dataframe has:
    - cycle number (sometimes the Z1 counter fucks up and just says 0)
    - If it is GC data: then calculate capacity
    
    If anything is wrong, it tries to fix it"""


    if df.experiment_mode == 1: #Then its GC
        if df['cycle number'].eq(0).all(): #If all cycle numbers are 0, then maybe Z1 counter was not iterated properly.
            LOG.info("We only found one cycle in '{}', and suspect this to be false. Checking now if there should be more cycles.".format(df.name))

            #We fix this by counting our own cycles.
            #Keeping track of signs of current (positive or negative) and cycle number
            prev_sign = True
            sign = True 
            cycle_number = 1
            new_cycle_indexes = []

            for i,current in df['<I>/mA'].items():

                if current > 0:
                    sign = True
                    df['charge'].at[i-1] = True
                elif current < 0:
                    sign = False
                    df['charge'].at[i-1] = False
                

                if prev_sign is False and sign is True:
                    #Changing from a discharge to a charge means new cycle
                    prev_sign = True
                    cycle_number += 1
                    #df['cycle number'].at[i-1] = cycle_number
                    new_cycle_indexes.append(i-1)
                    
                elif prev_sign is True and sign is False:
                    #Changing from a charge to a discharge
                    prev_sign = False
                    new_cycle_indexes.append(i-1)

                # In place editing of cycle number
                df['cycle number'].at[i] = cycle_number

            #Remove rows where a new experiment start (BatSmall has fucked datalogging here, where the current and voltage is the same as the prev step, but the capacity restarts)
            df.drop(new_cycle_indexes, axis = 0, inplace = True)

            if cycle_number > 1:
                LOG.info("Found {} cycles in {}".format(cycle_number, df.name))
            else:
                LOG.info("There seems to only be one cycle. Sorry for wasting your time.")

        else: #charge bool then isnt fixed
            for i,current in df['<I>/mA'].items():

                if current > 0:
                    sign = True
                    df['charge'].at[i-1] = True
                elif current < 0:
                    sign = False
                    df['charge'].at[i-1] = False



                



from ecdh.log import LOG
from ecdh.readers import (
    BatSmall as BS, 
    BioLogic as BL, 
    Neware as NA
)

from ecdh import utils

import os
import pandas as pd

def read(filepath):
    """
    Takes a filepath, finds out what file type it is, and reads it into two list of lists, fix this! maybe Cell object? or pd array?"""
    # Check input file and create proper data thereafter
    fn, ext = os.path.splitext(filepath)
    LOG.debug(f"Reading file: '{filepath}'")
    if ext == ".xlsx":
        df = NA.read_xlsx(filepath)
    elif ext == ".csv":
        df = NA.read_csv(filepath) #but this gives nested list with V/q data for each cycle.
    elif ext == ".mpt":
        df = BL.read_mpt(filepath)
    elif ext == ".customexport":
        df = BL.custom_EC_export(filepath)
    elif ext == ".txt":
        df = BS.read_txt(filepath)
    else:
        LOG.error(f"File format not supported: {ext}")
        LOG.error("Exiting..")
        exit()

    return df


def check_files(list):
    # Checks that files exist and then returns the ones that does.
    return_list = []
    for file in list:
        filename = file[0]
        if os.path.isfile(filename):
            return_list.append(file)
        else:
            LOG.error("File not found: '" + str(filename) + "' Please check that the correct path is typed in your input toml file. Skipping this file.")
    return return_list




import os as os
import matplotlib.pyplot as plt

from ecdh.log import LOG

from pandas.core.frame import DataFrame

import ecdh.readers as readers
import ecdh.utils as utils


"""
# Cell class specifications
1st argument = <string> // Path of file to be plotted
2nd argument = <string> // Active mass of whatever the data is made on
plot = <plot> object    // Describes which plot object to use. Normally this is the one specified above
start_cut = <int>       // Cuts specified number of cycles off the start of the data
"""



class Cell:
    def __init__(self, filename, am_mass, plot = None, specific_cycles = None):
        self.fn = filename
        self.am_mass = float(am_mass)
        self.color = plot.get_color()
        self.name = os.path.basename(filename)
        self.plotobj = plot
        self.axes = []
        self.mode_dict = {'0': 'Unspecified', '1': 'Galvanostatic', '2': "CyclicVoltammetry", '3': "Rest"}
        self.specific_cycles = specific_cycles
        self.GCdata = None
        self.CVdata = None


    def get_data(self):
        # Read input file
        self.df = readers.read(self.fn)
        LOG.debug("Data has been read successfully")


    def edit_CV_capacity(self):
        import numpy as np
        LOG.error("Cell.py/edit_CV_capacity has not been made! Creating data with only zeros.")
        self.CVdata_capacity = [(np.array([[0], [0]]), np.array([[0], [0]]))]


    def edit_CV(self):
        import numpy as np
        """Takes self.df and returns self.CVdata in the format:
        self.CVdata = [cycle1, cycle2, cycle3, ... , cycleN]
        cyclex = (chg, dchg)
        chg = np.array([[v1, v2, v3, ..., vn],[i1, i2, i3, ..., in]])
        So it is an array of the voltages and corresponding currents for a charge and discharge cycle in a cycle tuple in a list of cycles. 
        """
        if self.df.experiment_mode != 2: #If the data gathered isn't a cyclic voltammetry experiment, then this doesn't work!
            LOG.warning("File '{}' is not a CV file! It is a {} file.".format(self.fn, self.mode_dict[str(self.df.experiment_mode)]))
        else:
            self.CVdata = []

            #Remove all datapoints where mode != 2, we dont care about other data than CV here.
            index_names = self.df[self.df['mode'] != 2].index
            rawCVdata = self.df.drop(index_names)

            for cycle, subframe in rawCVdata.groupby('cycle number'):

                #Split into charge and discharge data
                chgdat = subframe[subframe['charge'] == True]
                dchgdat = subframe[subframe['charge'] == False]

                cycle = (np.array([chgdat['Ewe/V'], chgdat['<I>/mA']]), np.array([dchgdat['Ewe/V'], dchgdat['<I>/mA']]))
                self.CVdata.append(cycle)


    def edit_GC(self):
        import numpy as np
        """Takes self.df and returns self.GCdata in the format:
        self.GCdata = [cycle1, cycle2, cycle3, ... , cycleN]
        cyclex = (chg, dchg)
        chg = np.array([[q1, q2, q3, ..., qn],[v1, v2, v3, ..., vn]]) where q is capacity"""
        if self.df.experiment_mode != 1: #If the data gathered isn't a galvanostatic experiment, then this doesn't work!
            LOG.warning("File '{}' is not a GC file! It is a {} file.".format(self.fn, self.mode_dict[str(self.df.experiment_mode)]))
        else:
            self.GCdata = []

            #Remove all datapoints where mode != 1, we dont care about other data than GC here.
            index_names = self.df[self.df['mode'] != 1].index
            rawGCdata = self.df.drop(index_names)
            
            for cycle, subframe in rawGCdata.groupby('cycle number'):

                #Split into charge and discharge data
                chgdat = subframe[subframe['charge'] == True]
                dchgdat = subframe[subframe['charge'] == False]

                

                cycle = (np.array([chgdat['capacity/mAhg'], chgdat['Ewe/V']]), np.array([dchgdat['capacity/mAhg'], dchgdat['Ewe/V']]))
                self.GCdata.append(cycle)
            

    def edit_cyclelife(self):
        import numpy as np
        import pandas as pd
        #First make sure that we have either CV capacity data or galvanostatic
        if self.df.experiment_mode == 2:
            if not self.CVdata_capacity:
                LOG.error("in cell/edit_cyclelife, CVdata_capacity has not been made.")
                self.edit_CV_capacity()
        elif self.df.experiment_mode == 1:
            #If the GC data doesn't exist, make it.
            if not self.GCdata:
                self.edit_GC()

            #Make temporary dataholders
            tmpdat = []
            #loop through data and gather capacities
            for i, cycle in enumerate(self.GCdata):
                chg, dchg = cycle
                try:
                    tmpdat.append([i, chg[0][-1], dchg[0][-1]])
                except:
                    try:
                        tmpdat.append([i, chg[0][-1],0 ])
                    except:
                        try:
                            tmpdat.append([i, 0, dchg[0][-1]])
                        except:
                            tmpdat.append([i, 0, 0])

            self.cyclelifedata = pd.DataFrame(tmpdat, columns = ["cycle", "charge capacity/mAh", "discharge capacity/mAh"])

            self.cyclelifedata["coulombic efficiency"] = self.cyclelifedata["discharge capacity/mAh"] / self.cyclelifedata["charge capacity/mAh"]


    def edit_cumulative_capacity(self):
        """This function is used when you want the cumulative capacity for plotting potential and current versus capacity in a raw data plot"""
        LOG.debug("Calculating the cumulative capcity..")
        from scipy import integrate
        cumulative_capacity = integrate.cumtrapz(abs(self.df["<I>/mA"]), self.df["time/s"]/3600, initial = 0)
        self.df["CumulativeCapacity/mAh/g"] = cumulative_capacity/self.am_mass
        

    def treat_data(self, config):
        LOG.debug("Treating data")
        if config["start_cut"]:
            start_cut = int(config["start_cut"])
            # Trimming cycles
            if start_cut > len(self.discharges) or self.start_cut > len(self.charges):
                LOG.warning(f"Start cut is set to {self.start_cut} but the number of discharges is only {len(self.discharges)}. Cuts will not be made.")
            else:
                LOG.debug("Cutting off start cycles")
                self.discharges = self.discharges[self.start_cut:]
                self.charges = self.charges[self.start_cut:]

    def plot(self):
        if self.plotobj.vcplot:
            if self.df.experiment_mode == 2:
                self.edit_CV()
                self.plotobj.plot_CV(self)
            elif self.df.experiment_mode == 1:
                self.edit_GC()
                self.plotobj.plot_GC(self)

        if self.plotobj.qcplot:
            self.edit_cyclelife()
            self.plotobj.plot_cyclelife(self)

        if self.plotobj.rawplot:
            if self.plotobj.rawplot_capacity:
                self.edit_cumulative_capacity()

            self.plotobj.plot_raw(self)
        """
        if self.plotobj.vcplot == True:
            self.plot_cycles(self.plotobj)
        if self.plotobj.dqdvplot == True:
            self.plot_dqdv(self.plplotobjot)"""
        #if self.plotobj.dqdvplot == True and self.plotobj.vcplot == True:
        #    self.plot_cycles_dqdv(self.plotobj)


    def plot_cycles(self, plot):
        cmap = plot.colormap(self.color) #create colormap for fade from basic color
        # Slice to remove initial cycles Division by am mass to get specific capacity
        chg = self.charges
        dchg = self.discharges
        #Define lengths for use with colors
        Nc = len(chg)
        Nd = len(dchg)

        ax = plot.give_subplot() #Recieve correct subplot axis object from plot
        self.axes.append(ax)

        if type(plot.specific_cycles) != bool:
            for i, (charge_cycle, discharge_cycle) in enumerate(zip(chg, dchg)):
                if i in plot.specific_cycles: 
                    ax.plot(charge_cycle[1]/self.am_mass, charge_cycle[0],  c = plot.cycle_color(i))
                    ax.plot(discharge_cycle[1]/self.am_mass, discharge_cycle[0], label = make_label(i), c = plot.cycle_color(i))
                ax.legend()
        else:
            for i, (charge_cycle, discharge_cycle) in enumerate(zip(chg, dchg)):
                ax.plot(charge_cycle[1]/self.am_mass, charge_cycle[0],  c = cmap(i/Nc))
                ax.plot(discharge_cycle[1]/self.am_mass, discharge_cycle[0],  c = cmap(i/Nd))

            # Adding colorbar to plot
            sm = plt.cm.ScalarMappable(cmap=cmap, norm=plt.Normalize(vmin=0, vmax=Nd))
            sm._A = []
            plot.fig.colorbar(sm, ax=ax, label = "Cycle number")

        # Fixing title
        title = "VQ: " + os.path.basename(self.fn)
        ax.set_title(title)

        

    def plot_dqdv(self, plot):
        cmap = plot.colormap(self.color) #create colormap for fade from basic color
        # Slice to remove initial cycles Division by am mass to get specific capacity
        chg, dchg = make_dQdV(self.charges[self.start_cut:], self.discharges[self.start_cut:])
        #Define lengths for use with colors
        Nc = len(chg)
        Nd = len(dchg)

        ax = plot.give_subplot() #Recieve correct subplot axis object from plot
        self.axes.append(ax)

        if type(plot.specific_cycles) != bool:
            for i, (charge_cycle, discharge_cycle) in enumerate(zip(chg, dchg)):
                if i in plot.specific_cycles:
                    ax.plot(charge_cycle[1], charge_cycle[0],  c = plot.cycle_color(i))
                    ax.plot(discharge_cycle[1], discharge_cycle[0],  label = make_label(i), c = plot.cycle_color(i))
                ax.legend()
        else:
            for i, (charge_cycle, discharge_cycle) in enumerate(zip(chg, dchg)):
                ax.plot(charge_cycle[1], charge_cycle[0],  c = cmap(i/Nc))
                ax.plot(discharge_cycle[1], discharge_cycle[0],  c = cmap(i/Nd))
            # Adding colorbar to plot
            sm = plt.cm.ScalarMappable(cmap=cmap, norm=plt.Normalize(vmin=0, vmax=Nd))
            sm._A = []
            plot.fig.colorbar(sm, ax=ax, label = "Cycle number")

        # Fixing title
        title =  "dQ/dV: " + os.path.basename(self.fn)
        ax.set_title(title)
        ax.set(ylabel = "Potential [V]", xlabel = "dQ/dV [mAh/Vg]")

    def simplify(self, chg, dchg):
        # Takes two lists of complete cycle data and returns max capacity for each cycle.
        # chg = [cycle1, cycle2, cycle3 , ... , cycleN]
        # cycle1 = [[v1, v2, v3, ..., vn],[q1, q2, q3, ..., qn]]
        # This func basically just pulls qn from every cycle and creates a new array with it.
        import numpy as np

        charges = np.zeros(len(chg))
        discharges = np.zeros(len(dchg))
        for i, cycle in enumerate(chg):
            try:
                charges[i] = cycle[1][-1]
            except:
                charges[i] = 0
        for i, cycle in enumerate(dchg):
            try:
                discharges[i] = cycle[1][-1]
            except:
                discharges[i] = 0
        return charges, discharges



"""Data-readers for Neware"""
from ecdh.log import LOG
LOG.error("NEWARE READER HAS NOT BEEN MOVED OVER TO PD DATAFRAMES")
def read_csv(filepath):
    LOG.debug(f"Reading Neware CSV: '{filepath}'")
    import numpy as np
    #Open file
    with open(filepath, 'r', encoding = "ISO-8859-1") as f:
        #Read the first 2 lines which doesnt contain any data
        for i in range(2):
            f.readline()

        #Find what columns have our data
        Labels = f.readline().split(",") #Save the line of labels of which we are interested in
        V_col = 0
        mAh_col = 0
        for i,label in enumerate(Labels):
            if "Voltage(V)" in label:
                V_col = i
            if "Capacity(mAh)" in label:
                mAh_col = i

        #Read rest of data as list of lines
        data = f.readlines()
    
    #Index where in the file charges and discharges occur
    Chg_indxs = []
    DChg_indxs = []

    for i,line in enumerate(data):
        line_split = line.split(",")
        if "CC_Chg" in line_split[2]:
            Chg_indxs.append(i)
        elif "CC_DChg" in line_split[2]:
            DChg_indxs.append(i)

    charges = []
    discharges = []
    #Returns two arrays with Voltage and Capacity for the selected indexes of data.
    def slice_data(start, end, data):
        if end == -1:
            end = len(data)

        #Spawning data arrays
        start +=1
        end -=2
        voltages = np.zeros(end-start)
        capacities = np.zeros_like(voltages)
        for j,line in enumerate(data[start: end]):#Taking the slice of the data arr
            line = line.split(",")
            voltages[j] = float(line[V_col])
            capacities[j] = float(line[mAh_col])
        return voltages, capacities

    #Create one big list of all indexes in order to know which one is the next
    all_indxs = Chg_indxs + DChg_indxs
    all_indxs.sort()
    # Iterating over all indexes of charge start
    for i, indx in enumerate(Chg_indxs):
        i_all = all_indxs.index(indx) #Finding the corresponding item in list of all indexes
        start = indx   #Setting start of Charge to the charge index
        try:            #Setting end of charge to discharge index, or last index of data.
            end = all_indxs[i_all+1]
        except:
            end = len(data)-1
        #Getting the data slice, and putting it in charges list.
        voltages, capacities = slice_data(start, end, data)
        #print(voltages)
        charges.append((voltages, capacities))
    
    # Iterating over all indexes of discharge start
    for i, indx in enumerate(DChg_indxs):
        i_all = all_indxs.index(indx) #Finding the corresponding item in list of all indexes
        start = indx   #Setting start of Discharge to the discharge index
        try:            #Setting end of charge to charge index, or last index of data.
            end = all_indxs[i_all+1]
        except:
            end = len(data)-1
        #Getting the data slice, and putting it in charges list.
        voltages, capacities = slice_data(start, end, data)
        discharges.append((voltages, capacities))

            
    return charges, discharges


def read_xlsx(filename):
    import openpyxl as px
    import numpy as np

    # Open workbook
    wb = px.load_workbook(filename)
    # Get sheet
    sheet = wb[wb.sheetnames[3]]
    max_row = sheet.max_row-1 #-1 removes the first row with text
    # Spawn arrays for all raw data of columns
    status = []
    voltage = np.zeros(max_row)
    capacity = np.zeros(max_row)

    # Insert all data in arrays
    for i in range(max_row):
        voltage[i] = sheet.cell(row=i+2, column = 7).value
        capacity[i] = sheet.cell(row=i+2, column = 8).value
        status.append(sheet.cell(row=i+2, column=2).value)

    # Find where a charge and a discharge starts. Since charging occurs first, charges will be at even indexes of indx, eg: 0, 2, 4..
    indx =[] # Just a list to store where in the raw data a charge (even index) or discharge (odd index)
    for i in range(1,len(status)):
        if status[i] == "CC Chg" and status[i-1] != "CC Chg": # If status just changed to CC Chg, this is a charge start
            indx.append(i)
        elif status[i] == "CC DChg" and status[i-1] != "CC DChg": # If status just changed to CC DChg, this is a discharge start
            indx.append(i)
    indx.append(max_row) #Inserting last elem means we catch the end cycle!

    # Making lists for the cycles
    charges = []
    discharges = []

    # Inserting slices of raw data (corresponding to 1 cycle) into the list of charges
    for i in range(len(indx)-1): #-1 is to not run out of indexes when i+1
        if (i % 2) == 0: #index is even -> charge cycle
            charges.append((voltage[indx[i]:indx[i+1]], capacity[indx[i]:indx[i+1]])) #Inserting tuple of two arrays sliced to be one cycle of voltages and capacities.
        else: #Not even? Then its a discharge cycle
            discharges.append((voltage[indx[i]:indx[i+1]], capacity[indx[i]:indx[i+1]]))

    return charges, discharges


"""
Custom logger
Author: Knut Magnus Aasrud, from here: https://github.com/kmaasrud/doctor/blob/f6eea662de64dd2fa7a19548e71790ebd5084f3c/kodb/log.py
"""

import inspect
import shutil

LEVELS = {
    "DEBUG": 5,
    "D": 5,
    "INFO": 4,
    "I": 4,
    "SUCCESS": 3,
    "✓": 3,
    "WARNING": 2,
    "W": 2,
    "ERROR": 1,
    "E": 1,
    "CRITICAL": 0,
    "C": 0
}

class Message:
    def __init__(self, level):
        try:
            # Extract log level from string
            self.level = LEVELS[level.upper()]
        except (KeyError, AttributeError):
            # If key does not match, check if log_level is a valid int or else set the level to DEBUG
            self.level = level if type(level) == int and abs(level) < 5 else 4


    def set_level(self, level):
        self.__init__(level)


    def print_message(self, msg, level_name, *styles, file=None):
        if self.level >= LEVELS[level_name]:
            if file:
                # If there's room, then first print the name of the calling file right aligned
                left_align_len = len(level_name) + 2 + len(msg)
                print(self.right_align(f"({file})", left_align_len=left_align_len), end="\r")

            # Then print the message with the message level styled
            print("[" + self.style(level_name, *styles) + "]" + ":", msg)


    def debug(self, msg):
        from_frame = inspect.stack()[1]
        file = inspect.getfile(from_frame[0])
        self.print_message(msg, "DEBUG", "bold", file=file)


    def info(self, msg):
        self.print_message(msg, "INFO", "bold", "blue")


    def success(self, msg):
        self.print_message(msg, "SUCCESS", "bold", "green")


    def warning(self, msg):
        self.print_message(msg, "WARNING", "bold", "yellow")


    def error(self, msg):
        self.print_message(msg, "ERROR", "bold", "red")


    def critical(self, msg):
        from_frame = inspect.stack()[1]
        file = inspect.getfile(from_frame[0])
        self.print_message(msg, "C", "bold", "red", "underline", file=file)

    def style(self, text, *styles):
        code = {
            'red': '31',
            'green': '32',
            'yellow': '33',
            'blue': '34',
            'magenta': '35',
            'cyan': '36',
            'bright red': '91',
            'bright green': '92',
            'bright yellow': '93',
            'bright blue': '94',
            'bright magenta': '95',
            'bright cyan': '96',
            'bold': '1',
            'faint': '2',
            'italic': '3',
            'underline': '4',
            'blink': '5',
            'strike': '9'
        }

        for style in styles:
            text = "\033[" + code[style] + "m" + text + "\033[0m"

        return text

    def right_align(self, text, left_align_len=0):
        columns = shutil.get_terminal_size()[0]
        if left_align_len + len(text) < columns:
            return(text.rjust(columns))

LOG = Message("INFO")


import os
import sys
from datetime import date
from math import ceil, sqrt
from ecdh.log import LOG

import matplotlib as mpl
import matplotlib.colors as mcolors
import matplotlib.pyplot as plt
import matplotlib.ticker as mtick
from matplotlib.lines import Line2D

import numpy as np


"""
# Plot class specifications:
percentage = <bool>     // Only used when qc-plot = True and changes scale from specific capacity to percent of first cycle
qcplot = <bool>         // Displays capacity over cycle life plot
vcplot = <bool>         // Displays voltage curves (either CV or GC)
suptitle = <string>     // Changes the top title to whatever you put it to
ylabel = <string>       // Changes y-scale label on the vq plots
xlabel = <string>       // Changes x-scale label on the vq plots
numfiles = <int>        // Amount of files which is passed in the file list
"""

class Plot:
    def __init__(self, numfiles = 1, **kwargs):
        for key in kwargs:
            setattr(self, key, kwargs[key])
        self.taken_subplots = 0
        
        # Finding number of subplots
        if self.qcplot is True:
            self.subplots = 1
        else:
            self.subplots = 0

        if self.all_in_one is True:
            self.subplots+= 1
            if self.rawplot:
                self.subplots += 1
        else:
            if self.vcplot is True:
                self.subplots += numfiles
            
            if self.dqdvplot is True:
                self.subplots += numfiles
            
            if self.rawplot:
                self.subplots += numfiles

        
        # List of available colors
        self.colors =  ['tab:blue', 'tab:orange', 'tab:green', 'tab:red', 'tab:purple', 'tab:brown', 'tab:pink', 'tab:gray', 'tab:olive', 'tab:cyan' ]

        # Initiate figure and axes
        if self.subplots > 2: # Then two columns, or more.
            rows = int(sqrt(self.subplots))
            cols = ceil(self.subplots/rows)
            self.fig, self.axes = plt.subplots(nrows = rows, ncols = cols)
            self.axes = self.axes.reshape(-1)
        else:
            self.fig, self.axes = plt.subplots(nrows = self.subplots)
        self.fig.suptitle(str(date.today()))

        #Make sure self.axes is a list if it is only 1 element
        try:
            iter(self.axes)
        except:
            self.axes = [self.axes]
        
        for ax in self.axes:
            #ax.figure.set_size_inches(8.4, 4.8, forward = True)
            ax.set(
            title = 'Generic subplot title',
            ylabel = 'Potential [V]',
            xlabel = 'Specific Capacity [mAh/g]',
            #ylim = (2.5,5),
            #xlim = (0, 150),
            #xticks = (np.arange(0, 150), step=20)),
            #yticks = (np.arange(3, 5, step=0.2)),
            )
            ax.tick_params(direction='in', top = 'true', right = 'true')

        # If cycle life is to be plotted: Make the first subplot this.
        if self.qcplot == True:
            self.taken_subplots +=1
            # Dealing with percentage
            if self.percentage == True:
                ylabel = 'Capacity retention [%]'
                self.axes[0].yaxis.set_major_formatter(mtick.PercentFormatter(xmax = 1, decimals = 0))
            else:
                ylabel = 'Specific capacity [mAh/g]'
            
            self.axes[0].set(
                title = 'Cycle life',
                ylabel = ylabel,
                xlabel = 'Cycles'
            )

        if self.rawplot and self.all_in_one:
            self.axtwinx = None

    def give_subplot(self):
        ax = self.axes[self.taken_subplots]
        self.taken_subplots += 1 #Increment the subplots availability and give it to whomever.
        return ax


    def draw(self, save =False, show = True):
        
        if self.qcplot == True:
            # Get labels and handles for legend generation and eventual savefile
            handles, labels = self.axes[0].get_legend_handles_labels()
            handles.append(Line2D([0], [0], marker='o', color='black', alpha = 0.2, label = 'Charge capacity', linestyle=''))
            self.axes[0].legend(handles=handles)
            if type(self.specific_cycles) != bool:
                self.axes[0].scatter(self.specific_cycles, np.zeros(len(self.specific_cycles)), marker = "|")
            # Title also has to be adjusted
        
        # Makes more space.
        #self.fig.subplots_adjust(hspace=0.4, wspace=0.4)

        if self.all_in_one is True:
            plt.legend()

        # Save if True, If out path specified, save there.
        if save == True:
            savename = "CapRet"
            for label in labels:
                savename += "_" + label
            plt.savefig(savename)
        elif type(save) == str:
            plt.savefig(save)

        if show == True:
            plt.show()


    def get_color(self):
        give_color = self.colors[0]
        self.colors = self.colors[1:]
        return give_color
        

    def cycle_color(self, ncycle):
        colors = ['tab:blue', 'tab:orange', 'tab:green', 'tab:red', 'tab:purple', 'tab:brown', 'tab:pink', 'tab:gray', 'tab:olive', 'tab:cyan']
        return colors[self.specific_cycles.index(ncycle)]

    def colormap(self, color):
        color_hsv = np.asarray(mcolors.rgb_to_hsv(mcolors.to_rgb(color)))
        color_start = color_hsv.copy()
        color_end = color_hsv.copy()

        color_start[2] = 0
        color_end[2] = 1

        color1 = np.asarray(mcolors.hsv_to_rgb(color_start))*255
        color2 = np.asarray(mcolors.hsv_to_rgb(color_end))*255
        

        n = 100 #Numer of steps for colors
        diff = (color1-color2)/n
        colors = np.zeros((n,3))
        for i in range(len(colors)):
            colors[i] = color1 - i*diff
        colors = colors/255
        self.cmap = mpl.colors.LinearSegmentedColormap.from_list('colormap', colors)
        return self.cmap

    def plot_cyclelife(self, cellobj):
        """Takes a cell object and plots it in a cyclelife plot with either specific capacity or percentage on the y axis versus cycle number on the x axis"""
        data = cellobj.cyclelifedata

        norm_fact = data["discharge capacity/mAh"].iloc[0]

        if self.percentage == True: #Normalize capacities on the first cycle.
            data["discharge capacity/mAh"] = data["discharge capacity/mAh"] / norm_fact
            data["charge capacity/mAh"] = data["charge capacity/mAh"] / norm_fact
        else: # If not precentage, then it is specific
            data["discharge capacity/mAh"] = data["discharge capacity/mAh"] / cellobj.am_mass
            data["charge capacity/mAh"] = data["charge capacity/mAh"] / cellobj.am_mass

        ax = self.axes[0]   #Getting the right plot
        Nc = cellobj.cyclelifedata.count      #finding max number of cycles
        # Plot it
        if not cellobj.specific_cycles: #Not specific cycles? then plot all.
            ax.scatter(cellobj.cyclelifedata["cycle"]+1, cellobj.cyclelifedata["charge capacity/mAh"],color = cellobj.color, alpha = 0.2)
            ax.scatter(cellobj.cyclelifedata["cycle"]+1, cellobj.cyclelifedata["discharge capacity/mAh"],color = cellobj.color, label = cellobj.name)

        else: #There are specific cycles
            LOG.error("Specific cycles hasnt been implemented in lifecycle plot")
            #colorlist = self.colors
            #for i,cycle in enumerate(cellobj.cyclelifedata[3]):
            #    if cycle not in cellobj.specific_cycles:


    def plot_CV(self, cellobj):
        """Takes a cellobject and plots it in a CV plot (I/mA vs Ewe/V)"""
        LOG.debug("Running plot.py plot_CV")
        # Get subplot from main plotclass
        if self.all_in_one is False:
            ax = self.give_subplot()
            ax.set_title("CV: {}".format(os.path.basename(cellobj.fn)))
        else:
            ax = self.axes[0] if self.qcplot is False else self.axes[1]
            ax.set_title("Cyclic Voltammograms")
            
        #Placing it in a plot with correct colors
        self.insert_cycle_data(cellobj, ax, cellobj.CVdata)


        ax.set_ylabel("Current [mA]")
        ax.set_xlabel("Potential [V]")



    def plot_GC(self, cellobj):
        """Takes a cell dataframe and plots it in a GC plot (Ewe/V vs Cap)"""
        LOG.debug("Running plot.py plot_GC")

        # Get subplot from main plotclass
        if self.all_in_one is False:
            ax = self.give_subplot()
            ax.set_title("GC: {}".format(os.path.basename(cellobj.fn)))
        else:
            ax = self.axes[0] if self.qcplot is False else self.axes[1]
            ax.set_title("Galvanostatic Cycling")
            
        #Placing it in a plot with correct colors
        self.insert_cycle_data(cellobj, ax, cellobj.GCdata)

        
        ax.set_xlabel(r"Capacity [$\frac{mAh}{g}$]")
        ax.set_ylabel("Potential [V]")


    def insert_cycle_data(self, cellobj, ax, data):
        """
        Inserts the given data in the given axis with data from the cellobject
        Data must be on form 
        data = [cycle1, cycle2, cycle3, ... , cycleN]
        cyclex = (chg, dchg)
        chg = np.array([[x1, x2, x3, ..., xn],[y1, y2, y3, ..., yn]]) where x is usually either capacity or potential and y usually potential or current
        """
        # Generating colormap
        cmap = self.colormap(cellobj.color) #create colormap for fade from basic color
        #Define cycle amount for use with colors
        Nc = len(data)



        # Plot it
        if not cellobj.specific_cycles and Nc > 5: #Use colorbar if more than 4 cycles and no specific cycles.
            for i,cycle in enumerate(data):
                chg, dchg = cycle
                ax.plot(chg[0], chg[1], color = cmap(i/Nc))
                ax.plot(dchg[0], dchg[1], color = cmap(i/Nc))

            # Adding colorbar to plot
            sm = plt.cm.ScalarMappable(cmap=cmap, norm=plt.Normalize(vmin=0, vmax=Nc))
            sm._A = []
            if self.all_in_one is True:
                self.fig.colorbar(sm, ax=ax, label = "Cycle number for {}".format(cellobj.name))
            else:
                self.fig.colorbar(sm, ax=ax, label = "Cycle number")

        else: #There are either specific cycles or <=5 cycles in the data

            colorlist = self.colors
            if cellobj.specific_cycles:
                for i,cycle in enumerate(data):
                    if i in cellobj.specific_cycles:
                        color = colorlist[0]
                        colorlist = colorlist[1:]

                        chg, dchg = cycle
                        ax.plot(chg[0], chg[1], color = color, label = "Cycle {}".format(i)) #This is the charge cycle
                        ax.plot(dchg[0], dchg[1], color = color) #1 is discharge
            else:
                for i,cycle in enumerate(data):

                    color = colorlist[0]
                    colorlist = colorlist[1:]

                    chg, dchg = cycle
                    ax.plot(chg[0], chg[1], color = color, label = "Cycle {}".format(i)) #This is the charge cycle
                    ax.plot(dchg[0], dchg[1], color = color) #1 is discharge

    def plot_raw(self,cellobj):
        """Takes a cellobject and plots it in a raw data plot (I/mA and Ewe/V vs time/s)"""
        LOG.debug("Running plot.py plot_raw")
        # Get subplot from main plotclass
        if self.all_in_one is False:
            ax = self.give_subplot()
            ax.set_title("raw data: {}".format(os.path.basename(cellobj.fn)))
            #Initiate twin ax
            ax2 = ax.twinx()
            #self.axes.append(ax2)
        else:
            if self.qcplot and self.rawplot:
                ax = self.axes[2]
            elif self.qcplot or self.rawplot: 
                ax = self.axes[1]
            else:
                ax = self.axes[0]
            #Initiate twin axis, only if it doesnt exist aleady
            if not self.axtwinx:
                self.axtwinx = ax.twinx()
            ax2 = self.axtwinx

            ax.set_title("Raw data")
            
        #Placing it in a plot
        if self.rawplot_capacity:
            x = cellobj.df["CumulativeCapacity/mAh/g"]
            ax.set_xlabel(r"Cumulative Capacity [$\frac{mAh}{g}$]")
        else:
            x = cellobj.df["time/s"]/3600
            ax.set_xlabel("Time [h]")

        ax.plot(x, cellobj.df["Ewe/V"], color = cellobj.color)
        ax2.plot(x, cellobj.df["<I>/mA"], color = cellobj.color, linestyle = "dotted")


        ax.set_ylabel("Potential [V]")
        ax2.set_ylabel("Current [mA]")
        


# -*- coding: utf-8 -*-

"""This file is ran when user calls ecdh"""

from ecdh.log import LOG
LOG.set_level("DEBUG")
from ecdh.readers import check_files

from ecdh.make_toml import make_toml
from ecdh.plot import *
from ecdh.cell import *

import toml
import sys

def run():

    if len(sys.argv) < 3: #Then no folder is specified, look for toml in local folder.
        if os.path.isfile("./ecdh.toml"):
            path = "./ecdh.toml"
        else:
            LOG.error("Could not find an ecdh.toml file in the current directory! \nOptions: \n1. Run ecdh with the argument 'run' followed by the path of your .toml file. \n2. Initiate toml file in this directory with the init argument.")
            sys.exit()
    elif os.path.isfile(sys.argv[2]):# File was inserted Read toml config
        path = sys.argv[2]
    else:
        LOG.error("Cannot find the .toml configuration file!")
        sys.exit()

    toml_str = open(path, "r").read()
    config = toml.loads(toml_str)
    settings = config["settings"]

    # Check that files are found
    files = check_files(config["files"])
    LOG.success("Running ECDH: Found {} datafiles!".format(len(files)))

    # Define plot specifications
    plot = Plot(numfiles=len(files), **settings)


    # Run the data reading + plot generation
    cells = []
    for f in files:
        cell = Cell(f[0], f[1], plot=plot)
        cell.get_data()
        #cell.edit_GC()
        #cell.treat_data(settings)
        cell.plot()
        #cells.append(cell)



    plot.draw()

def main():
    if len(sys.argv) < 2:
        run()
    elif sys.argv[1] == "init":
        make_toml("./")
    elif sys.argv[1] == "run":
        run()

if __name__ == "__main__":
    main()

    



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name = 'ecdh',
    packages = ['ecdh'],
    include_package_data = True,
    version = '0.1.3',     # Remember to update on new version
    license='GNU General Public License v3.0',        
    description = 'ElectroChemical Data Handler',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author = 'Amund Raniseth',                   
    author_email = 'amund.raniseth@gmail.com',      
    url = 'https://github.com/amundmr/ecdh',   
    download_url = 'https://github.com/amundmr/ecdh/archive/refs/tags/v0.1.3.tar.gz',    # Remember to update this with new versions
    keywords = ['Electrochemical', 'battery', 'cell', 'datareader', 'dataplotter', 'reader', 'plotter'],
    install_requires=[
            'matplotlib',
            'numpy',
            'pandas',
            'toml',
            'scipy',
        ],
    classifiers=[
        'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',      # Define that your audience are developers
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3',      #Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    entry_points={
        "console_scripts": [
            "ecdh=ecdh.__main__:main",
        ]
    },
)



import os


def make_dQdV(chg, dchg, nbins = 200):
    # chg = [cycle1, cycle2, cycle3 , ... , cycleN]
    # cycle1 = [[v1, v2, v3, ..., vn],[q1, q2, q3, ..., qn]]
    import numpy as np

    bins = np.linspace(chg[0][0][0],chg[0][0][-1],nbins)

    chg_new = []
    dchg_new = []
    for cyc, dcyc in zip(chg,dchg):
        cyc_dqdvs = []
        dcyc_dqdvs = []
        for i in range(len(bins)-1):
            #print("Bin start: %.2f stop: %.2f" %(bins[i], bins[i+1]))
            start_indx = (np.abs(cyc[0] - bins[i])).argmin()
            end_indx = (np.abs(cyc[0] - bins[i+1])).argmin()
            #print("start indx: %i, stop indx: %i" % (start_indx, end_indx))
            cyc_dQdV = (cyc[1][end_indx]-cyc[1][start_indx]) / (cyc[0][end_indx] - cyc[0][start_indx])

            start_indx = (np.abs(dcyc[0] - bins[i])).argmin()
            end_indx = (np.abs(dcyc[0] - bins[i+1])).argmin()
            #print("start indx: %i, stop indx: %i" % (start_indx, end_indx))
            dcyc_dQdV = (dcyc[1][end_indx]-dcyc[1][start_indx]) / (dcyc[0][end_indx] - dcyc[0][start_indx])

            cyc_dqdvs.append(cyc_dQdV)
            dcyc_dqdvs.append(dcyc_dQdV)

        chg_new.append((bins[:-1],(np.asarray(cyc_dqdvs))))
        dchg_new.append((bins[:-1],(np.asarray(dcyc_dqdvs))))

    
    return chg_new, dchg_new

def make_label(ncycle):
    ordinal = lambda n: "%d%s" % (n,"tsnrhtdd"[(n//10%10!=1)*(n%10<4)*n%10::4])
    return str(ordinal(ncycle)) + " cycle"